# $Progeny: base.py,v 1.80 2002/06/12 17:01:43 branden Exp $

import os
import string
import sys

import pgi.common

from pgi.common import TRUE, FALSE

LIVE = pgi.common.get_live()
LOGFILE = pgi.common.get_logfile()
MESSAGES = LIVE + "/share/installer/messages"

# TODO: Make these functions paranoid!  Don't just assume that everything is
# hunky-dory and unconditionally return TRUE!

def _write_fstab_line(fstab, device, mntpnt, fstype, options, dumpno,
                           passno):
    fstab.write("%s\t%s\t%s\t%s\t%d\t%d\n" % (device, mntpnt, fstype,
                                              options, dumpno, passno))

def get_messages(type):
    retval = None

    if os.path.exists(MESSAGES):
        druidtext = open(MESSAGES)
        state = "none"
        druid_texts = {}
        for line in druidtext.readlines():
            if line[0] == "<":
                if line[1] == "/":
                    state = "none"
                else:
                    state = line[1:string.find(">", line) - 1]
                continue

            if not druid_texts.has_key(state):
                druid_texts[state] = ""
            druid_texts[state] = druid_texts[state] + line

        druidtext.close()

        if druid_texts.has_key(type):
            retval = druid_texts[type]

    return retval

def mount(mounts, target, callback = None, cbdata = None):
    mntpnts = mounts.keys()
    mntpnts.sort()

    for mntpnt in mntpnts:
        (device, fstype) = mounts[mntpnt]
        pgi.common.msg("Mounting %s filesystem on %s at %s." % (fstype, device, mntpnt))
        if mntpnt == "/":
            # Write this to the initrd's fstab so stage0.sh will have
            # no problems remounting /target read-only.
            fstab = open("/etc/fstab", "a")
            fstab.write("%s %s %s defaults 0 0\n" % (device, target, fstype))
            fstab.close()

            command = "mkdir -p %s" % (target,)
            pgi.common.run_command(command, 1, callback, cbdata)
            command = "mount -t %s %s %s" % (fstype, device, target)
        else:
            command = "mkdir -p %s" % (target + mntpnt,)
            pgi.common.run_command(command, 1, callback, cbdata)
            command = "mount -t %s %s %s" % (fstype, device, target + mntpnt)

        pgi.common.run_command(command, 1, callback, cbdata)

    return TRUE

# XXX: at present, nothing uses this function
def unmount(mounts, target):
    mntpnts = mounts.keys()
    mntpnts.sort()

    for mntpnt in mntpnts:
        (device, fstype) = mounts[mntpnt]
        if mntpnt != target:
            command = "umount %s" % (mntpnt,)
        else:
            continue

        pgi.common.run_command(command)

    return TRUE

def install_packages(suite, mirror, target, \
                     extra_pkg_filename = None, \
                     unwanted_pkg_filename = None, \
                     callback = None, cbdata = None):
    if os.path.exists("%s/debian" % (LIVE,)):
        mirror = "file://%s/debian" % (LIVE,) # LIVE already has a prefixed /
    pgi.common.msg("Using %s mirror.\n" % (mirror,))

    # this doesn't seem to help with configlet-frontends's postinst
    #del os.environ["PYTHONHOME"]

    if os.environ["PGIDEBUG"] == "noisy":
        debug_option = "-x"
    else:
        debug_option = ""

    if extra_pkg_filename:
        try:
            extra_pkg_file = open(extra_pkg_filename)
            extra_pkg_data = extra_pkg_file.read()
            extra_pkg_file.close()

            extra_pkgs = string.split(extra_pkg_data)
            extra_pkg_arg = "--include=%s" % (string.join(extra_pkgs, ","),)
            pgi.common.msg("Including extra packages: %s\n" % (extra_pkg_arg,))
        except IOError, e:
            pgi.common.msg("Cannot include extra packages; error %s.\n" % (e,))
    else:
        extra_pkg_arg = ""
        pgi.common.msg("No extra packages identified for inclusion.\n")

    if unwanted_pkg_filename:
        try:
            unwanted_pkg_file = open(unwanted_pkg_filename)
            unwanted_pkg_data = unwanted_pkg_file.read()
            unwanted_pkg_file.close()

            unwanted_pkgs = string.split(unwanted_pkg_data)
            unwanted_pkg_arg = "--exclude=%s" % (string.join(unwanted_pkgs, ","),)
            pgi.common.msg("Excluding unwanted packages: %s\n" % (unwanted_pkg_arg,))
        except IOError, e:
            pgi.common.msg("Cannot exclude unwanted packages; error %s.\n" % (e,))
    else:
        unwanted_pkg_arg = ""
        pgi.common.msg("No unwanted packages identified for exclusion.\n")

    command = "%s/sbin/bash %s %s/sbin/debootstrap %s %s %s %s %s" \
              % (LIVE, debug_option, LIVE, extra_pkg_arg, unwanted_pkg_arg,
                 suite, target, mirror)
    exitcode = pgi.common.run_command(command, 1, callback, cbdata)
    if exitcode != 0:
        return FALSE

    if os.path.exists("%s/share/installer/sources.list.dist" % (LIVE,)):
        if os.path.isdir("%s/etc/apt" % (target,)):
            command = "cp %s/share/installer/sources.list.dist %s/etc/apt/sources.list" % (LIVE, target)
            pgi.common.run_command(command, 1, callback, cbdata)

    if not os.path.exists("%s/tmp" % (target,)):
        os.mkdir("%s/tmp" % (target,))

    if os.path.isdir("%s/tmp" % (target,)):
        os.chmod("%s/tmp" % (target,), 01777)
    else:
        pgi.common.msg("%s/tmp is not a directory!" % (target,))

    return TRUE

def mkfstab(mounts, swaps, target):
    mntpnts = mounts.keys()
    mntpnts.sort()

    fstab = open("%s/etc/fstab" % (target,), "w")
    fstab.write("""
# /etc/fstab: static file system information.
#
# <file system> <mount point> <type> <options> <dump> <pass>
""")

    for mntpnt in mntpnts:
        (device, fstype) = mounts[mntpnt]
        if mntpnt == "/":
            _write_fstab_line(fstab, device, mntpnt, fstype,
                              "defaults,errors=remount-ro", 0, 1)
        else:
            _write_fstab_line(fstab, device, mntpnt, fstype,
                              "defaults", 0, 2)
    for swap in swaps:
        _write_fstab_line(fstab, swap, "none", "swap", "sw", 0, 0)

    _write_fstab_line(fstab, "proc", "/proc", "proc", "defaults", 0, 0)
    _write_fstab_line(fstab, "/dev/fd0", "/floppy", "auto",
                      "defaults,user,noauto", 0, 0)
    _write_fstab_line(fstab, "/dev/cdrom", "/cdrom", "iso9660",
                      "defaults,ro,user,noauto", 0, 0)
    fstab.close()

    return TRUE

def install_kernel(target, callback = None, cbdata = None):
    command = "(cd %s/kernel && tar cf - .) | (cd %s && tar xvf -)" % (LIVE, target)
    pgi.common.run_command(command, 1, callback, cbdata)
    command = "chroot %s /sbin/depmod -a" % (target,)
    pgi.common.run_command(command, 1, callback, cbdata)

    return TRUE

if __name__ == "__main__":
    pass

# vim:ai:et:sts=4:sw=4:tw=0:
