# $Progeny: installer-text.sh,v 1.114 2002/06/12 19:44:55 branden Exp $

# Syntax: calc_textbox_size filename
#  e.g., calc_textbox_size /tmp/welcome-msg
calc_textbox_size() {
    if [ -e "$1" ]; then
        rows=$(( $(cat "$1" | wc -l) + 4 ))
        if [ $rows -lt 5 ]; then
            rows=5
        elif [ $rows -gt 19 ]; then
            rows=19
        fi
    fi
    # columns fixed at 70 because the title might be wider than the longest
    # line of the file
    echo $rows 70
}

# Syntax: check_partition disk_partition_device
#   e.g., check_partition /dev/hda1
check_partition () {
    if [ -z "$1" ]; then
        pgilog "Internal error: check_partition() called without an argument."
        return 2
    else
        # XXX: erm, would be nice to support ext3, reiserfs, etc.
        # XXX: erm, not just nice, required in the case of ia64 (EFI)
        for type in ext2 vfat; do
            if mount -t $type -o ro $1 /mnt >> $LOGFILE 2>&1; then
                umount $1 >> $LOGFILE 2>&1
                PARTTYPE=$type
                return 0
            fi
        done
        return 1
    fi;
}

# Syntax: build_partition_selection_menu
build_partition_selection_menu () {
    # build list of partitions available for selection
    PARTITION_LIST_MENU=
    for PARTITION in $PARTITION_LIST; do
        if [ -n "$PARTITION_LIST_MENU" ]; then
            PARTITION_LIST_MENU="$PARTITION_LIST_MENU $PARTITION ''"
        else
            PARTITION_LIST_MENU="$PARTITION ''"
        fi
    done

    # build list of disks available for viewing
    DISK_LIST_VIEW_MENU=
    for DISK in $DISK_LIST; do
        # don't change the text without changing the grep and sed commands
        # elsewhere in the code
        if [ -n "$DISK_LIST_VIEW_MENU" ]; then
            DISK_LIST_VIEW_MENU="$DISK_LIST_VIEW_MENU 'View partition table of $DISK' ''"
        else
            DISK_LIST_VIEW_MENU="'View partition table of $DISK' ''"
        fi
    done

    DIALOG_MENU_LIST="$PARTITION_LIST_MENU $DISK_LIST_VIEW_MENU"
    pgilog "\$DIALOG_MENU_LIST is \"$DIALOG_MENU_LIST\".";
}

# Syntax: display_partition_table disk_device
#  e.g., display_partition_table /dev/hda
display_partition_table () {
    if [ -z "$1" ]; then
        pgilog "Internal error: display_partition_table() called without an argument."
        return 2
    else
        TEMPFILE=$(tempfile)
        parted $1 print > $TEMPFILE
        $DIALOG --backtitle "PGI Debian Installer" \
            --title "Partition Report for $1" --textbox $TEMPFILE $(calc_textbox_size $TEMPFILE)
        rm -f $TEMPFILE
    fi;
}

echo "Starting PGI Debian text-mode installer..."

# give the user time to read the above before the dialogs begin
sleep 2

DIALOG=$LIVE/bin/dialog
PARTED=$LIVE/bin/parted

EXTRA_PACKAGES=$LIVE/share/installer/extra_packages
UNWANTED_PACKAGES=$LIVE/share/installer/unwanted_packages

pgilog "Prompting mode is \"$PROMPTING\"."
pgilog "Using Linux kernel $KERNEL_VERSION."

echo "lspci output:" >> $LOGFILE
lspci >> $LOGFILE
echo "*******************************************************" >> $LOGFILE

echo "lspci -n output (no different from above if PCI list not compiled into kernel):" >> $LOGFILE
lspci -n >> $LOGFILE
echo "*******************************************************" >> $LOGFILE

pgilog "Dumping dmesg to $DMESGLOG."
dmesg >> $DMESGLOG

for COMMAND in $DIALOG $PARTED; do
    while :; do
        pgilog -n "Checking for command $COMMAND..."
        if command -v $(basename $COMMAND) > /dev/null 2>&1; then
            pgilog "found."
            break
        else
            pgilog "not found.  Spawning shell."
            echo "Error in installer environment: can't find $COMMAND in $PATH."
            echo "Here's a shell prompt so you can set things right."
            envsh
        fi
    done
done

USER_HAPPY=
INSTALL_BOOTLOADER=
CANCEL=

# ensure no filesystems from a previous install attempt are still mounted
unmount_targets

# sorry, folks, gotta turn off debugging for dialog's sake
set +x

# get welcome message
python $LIVE/share/installer/get-message.py start | sed 's/"/\\"/g' \
    > /tmp/welcome-msg
if [ ! -s /tmp/welcome-msg ]; then
    cat > /tmp/welcome-msg << EOF
Welcome to the PGI Debian GNU/Linux Installer.

There are four parts to the installation process:
  First, disks are partitioned.
  Second, the boot loader is configured (if desired).
  Third, the system packages are installed.
  Fourth, the boot loader is set up (if desired).
  Finally, the new system is entered and configured.

Be sure to back up any important data before installation.

Thank you for using Debian GNU/Linux!
EOF
fi

# welcome
pgilog "Prompting user with welcome screen."
$DIALOG --no-cancel --cr-wrap --backtitle "PGI Debian Installer" \
    --title "Welcome to Debian GNU/Linux" \
    --textbox /tmp/welcome-msg $(calc_textbox_size /tmp/welcome-msg)

pgilog "Prompting user with proceed screen."
$DIALOG --no-cancel --cr-wrap --backtitle "PGI Debian Installer" \
    --title "Start Installation Procedure" \
    --yesno "Proceed with installation of Debian GNU/Linux?" 5 70
# check for cancellation
if [ $? -ne 0 ]; then
    CANCEL=yes
    break
fi

# get list of available disk devices from parted
DISK_LIST=$(list-devices --disks)
pgilog "\$DISK_LIST is \"$DISK_LIST\"."

if [ -z "$DISK_LIST" ]; then
    pgilog "Prompting user that no disks could be found."
    $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
        --title "Error in Installer Environment" \
        --msgbox \
        "GNU parted cannot find any disks on the system.\n\nThe Linux kernel used by this installer either does not support your hardware, or there are no hard drives installed.\n\nCancelling installation." 10 70
    CANCEL=yes
fi

while [ -z "$USER_HAPPY" -a -z "$CANCEL" ]; do
    ############################################################################
    # partition disks
    ############################################################################

    DIALOG_MENU_LIST=
    for DISK in $DISK_LIST; do
        DIALOG_MENU_LIST="$DIALOG_MENU_LIST $DISK ''"
    done

    DIALOG_MENU_LIST="$DIALOG_MENU_LIST 'Done Partitioning' ''"

    while :; do
        pgilog "Prompting user with list of disks to partition."
        eval $DIALOG --no-cancel --cr-wrap --backtitle \"PGI Debian Installer\" \
            --title \"Select Disk to Partition\" --menu \
            \"Which disk do you want to partition?\\\n\\\nThe GNU parted program will be launched to partition the selected disk.  For help with GNU parted, simply type \'help\' at its prompt.\\\n\\\nDon\'t forget to create filesystems on the partition\(s\) you will be using \(except for swap partitions, of course\).\" 17 70 4 \
            $DIALOG_MENU_LIST \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
        DISK=$(cat /tmp/prompt)
        pgilog "User chose \"$DISK\"."

        if [ "$DISK" = "Done Partitioning" ]; then
            pgilog "User is done partitioning."
            # get list of known partitions from parted; this is newline-delimited
            PARTITION_LIST=$(list-devices)
            pgilog "\$PARTITION_LIST is:"
            pgilog "$PARTITION_LIST"
            # extra echo needed to work around bug in busybox sh or test
            NUM_PARTITIONS=$(echo "$PARTITION_LIST" | wc -l)
            if [ $NUM_PARTITIONS -lt 2 ]; then
                pgilog "Prompting user that there are not enough partitions (only $NUM_PARTITIONS)."
                $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
                    --title "Insufficient Number of Disk Partitions" --msgbox \
                    "GNU parted cannot find enough disk partitions on the system to support an installation of Debian GNU/Linux.\n\nAt least two partitions are required; one for the root filesystem and one for swap space.\n\nPlease create some partitions." 11 70
                # check for cancellation
                if [ $? -ne 0 ]; then
                    CANCEL=yes
                    break 2
                fi
            else
                break
            fi
        else
            pgilog "Running \"$PARTED --interactive $DISK\"."
            clear
            $PARTED --interactive $DISK
        fi
    done

    ############################################################################
    # associate partitions with mount points; we know there are at least two

    # choose a swap partition
    build_partition_selection_menu
    while :; do
        pgilog "Prompting user for swap device."
        eval $DIALOG --no-cancel --cr-wrap --backtitle \"PGI Debian Installer\" \
            --title \"Select a Partition to Use As the Swap Device\" \
            --menu \
            \"Choose a partition to use as your swap partition.\" 19 70 4 \
            $DIALOG_MENU_LIST \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break 2
        fi
        # check for partition viewing
        if grep -q View /tmp/prompt; then
            pgilog "User wants to \"$(cat /tmp/prompt)\"."
            # kludgey!
            DISK=$(sed 's/View partition table of //' < /tmp/prompt)
            display_partition_table $DISK
        else
            break
        fi
    done
    SWAPPART=$(cat /tmp/prompt)
    pgilog "User said \"$SWAPPART\" is the swap device."

    # eliminate this partition from future choices
    PARTITION_LIST=$(echo "$PARTITION_LIST" | grep -v "^$SWAPPART\$")
    pgilog "\$PARTITION_LIST is now:"
    pgilog "$PARTITION_LIST"

    # choose a root partition
    build_partition_selection_menu
    while :; do
        pgilog "Prompting user for root filesystem."
        eval $DIALOG --no-cancel --cr-wrap --backtitle \"PGI Debian Installer\" \
            --title \"Select a Partition to Use As the Filesystem Root\" \
            --menu \"Choose a partition to use as your root partition.\" 19 70 4 \
            $DIALOG_MENU_LIST \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break 2
        fi
        # check for partition viewing
        if grep -q View /tmp/prompt; then
            pgilog "User wants to \"$(cat /tmp/prompt)\"."
            # kludgey!
            DISK=$(sed 's/View partition table of //' < /tmp/prompt)
            display_partition_table $DISK
        else
            break
        fi
    done
    # for /etc/fstab, GRUB's menu.lst, and other purposes
    ROOTPART=$(cat /tmp/prompt)
    pgilog "User said \"$ROOTPART\" is the root filesystem."

    if ! check_partition $ROOTPART; then
        pgilog "Prompting user that $ROOTPART is not formatted."
        $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
            --title "Disk Partition Not Formatted" --msgbox \
            "The partition $ROOTPART cannot be used as the root filesystem because it is not formatted.\n\nThe disk partitioning procedure will restart so that you may correct any errors." 9 70
        # no need to check for cancellation, we're going back to the top
        continue
    fi
    MNTPNTS="/ $ROOTPART ext2"
    # for grub-install
    ROOTDISK=$(echo $ROOTPART | sed 's/[[:digit:]]*$//')
    pgilog "Using \"$ROOTDISK\" as the root disk."

    # eliminate this partition from future choices
    PARTITION_LIST=$(echo "$PARTITION_LIST" | grep -v "^$ROOTPART\$")
    pgilog "\$PARTITION_LIST is now:"
    pgilog "$PARTITION_LIST"

    MOUNTPOINT_LIST=$(printf '/home\n/tmp\n/usr\n/usr/local\n/var\n/var/mail\n/var/spool\n/var/tmp\nswap\n/boot/efi\nNONE')

    # are there any partitions left? If so, offer a mount point for chosen partitions.
    while [ -n "$PARTITION_LIST" ]; do
        build_partition_selection_menu
        pgilog "Prompting user for partitions for other mount points."
        eval $DIALOG --cr-wrap --backtitle \"PGI Debian Installer\" \
            --title \"Select a Partition\" \
            --menu \"Choose a partition to mount at another mount point.\" 19 70 4 \
            $DIALOG_MENU_LIST \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
        # check for partition viewing
        if grep -q View /tmp/prompt; then
            pgilog "User wants to \"$(cat /tmp/prompt)\"."
            # kludgey!
            DISK=$(sed 's/View partition table of //' < /tmp/prompt)
            display_partition_table $DISK
            continue
        fi

        PARTITION=$(cat /tmp/prompt)
        pgilog "User said \"$PARTITION\" is to be mounted."

        # build menu of mount points
        DIALOG_MENU_LIST=
        for  MOUNTPOINT in $MOUNTPOINT_LIST; do
            if [ -n "$DIALOG_MENU_LIST" ]; then
                DIALOG_MENU_LIST="$DIALOG_MENU_LIST $MOUNTPOINT ''"
            else
                DIALOG_MENU_LIST="$MOUNTPOINT ''"
            fi
        done

        pgilog "Prompting user for mount point of $PARTITION."
        eval $DIALOG --cr-wrap \
            --backtitle \"PGI Debian Installer\" \
            --title \"Select Mount Point for $PARTITION\" \
            --menu \
            \"Please select a mount point for $PARTITION.  Choose from any of the mount points below.  You may select 'swap' to set up supplementary swap partitions, or 'NONE' to have Debian GNU/Linux ignore the partition completely.\" 19 70 9 \
            $DIALOG_MENU_LIST \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            continue
        fi

        SELECTED_MOUNTPOINT=$(cat /tmp/prompt)
        pgilog "User chose mount point of \"$SELECTED_MOUNTPOINT\" for \"$PARTITION\"."
        case "$SELECTED_MOUNTPOINT" in
        NONE)
            ;;
        swap)
            SWAPPART="$SWAPPART $SELECTED_MOUNTPOINT"
            ;;
        *)
            if check_partition $PARTITION; then
                MNTPNTS="$MNTPNTS $SELECTED_MOUNTPOINT $PARTITION $PARTTYPE"
                # eliminate this mount point from future choices
                MOUNTPOINT_LIST=$(echo "$MOUNTPOINT_LIST" | grep -v "^$SELECTED_MOUNTPOINT\$")
            else
                pgilog "Prompting user that $PARTITION is not formatted."
                $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
                    --title "Disk Partition Not Formatted" --msgbox \
                    "The partition $PARTITION cannot be used as a mount point because it is not formatted." 6 70
                # no need to check for cancellation; just jump back to the
                # top of this loop
                continue
            fi
            ;;
        esac

        if [ "$SELECTED_MOUNTPOINT" = "/boot/efi" ]; then
            EFIPART="$SELECTED_MOUNTPOINT"
        fi

        # eliminate this partition from future choices
        PARTITION_LIST=$(echo "$PARTITION_LIST" | grep -v "^$PARTITION\$")
        pgilog "\$PARTITION_LIST is now:"
        pgilog "$PARTITION_LIST"
    done

    # XXX: maybe also report partition type.  Maybe.
    BASIC_TEXT=$(printf "You have selected $ROOTPART as your root filesystem.\\\n\\\nYour swap partition(s) are: $SWAPPART")
    EXTRA_TEXT=
    if [ -n "$MNTPNTS" ]; then
        set -- $MNTPNTS
        while [ -n "$1" ]; do
            if [ "$1" != "/" ]; then
                if [ -z "$EXTRA_TEXT" ]; then
                    EXTRA_TEXT="\n\nYou also have specified mount points for the following partitions:\n\n"
                fi
                EXTRA_TEXT=$(printf "${EXTRA_TEXT}$1\t$2\\\n")
            fi
            shift 3
        done
    fi

    pgilog "Prompting user to proceed with installation."
    # if there is a lot of text to display, use a separate message box for the
    # partitioning report
    if [ $(echo "$EXTRA_TEXT" | wc -l) -le 6 ]; then
        $DIALOG --no-cancel --cr-wrap --backtitle "PGI Debian Installer" \
            --title "Proceed with Installation?" --menu \
            "$BASIC_TEXT$EXTRA_TEXT" 19 71 3 \
            "proceed" "Continue installation of Debian" \
            "re-partition" "Change partitioning, swap devices, and mount points" \
            "cancel" "Cancel installation of Debian" \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
    else
        $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
            --title "Disk Partitioning Report" --msgbox \
            "$BASIC_TEXT$EXTRA_TEXT" 19 70
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
        $DIALOG --no-cancel --cr-wrap --backtitle "PGI Debian Installer" \
            --title "Proceed with Installation?" --menu \
            "Would you like to continue with the installation process?\n\nThis is your last chance to cancel before irreversible changes are made to your system." 13 71 3 \
            "proceed" "Continue installation of Debian" \
            "re-partition" "Change partitioning, swap devices, and mount points" \
            "cancel" "Cancel installation of Debian" \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
    fi

    # reset CANCEL before popping the question
    CANCEL=
    CHOICE=$(cat /tmp/prompt)
    pgilog "User chose to \"$CHOICE\"."

    case "$CHOICE" in
    proceed)
        USER_HAPPY=yes
        ;;
    re-partition)
        ;;
    cancel)
        CANCEL=yes
        ;;
    esac
done

if [ -z "$CANCEL" ]; then
    # XXX: arch-specific tests shouldn't be done here
    if [ "$ARCH" = "ia64" ]; then
        INSTALL_BOOTLOADER=yes
    else
        pgilog "Prompting user about if/how the boot loader should be installed."
        $DIALOG --no-cancel --cr-wrap --backtitle "PGI Debian Installer" \
            --title "Select Desired Boot Method" --menu \
            "You have the option of installing a boot loader to the master boot record of your primary disk, or to the root partition on that disk.  Alternatively, you can skip the installation of a boot loader and use your own, alternative means of booting the system." 15 70 3 \
            "mbr" "Install boot loader to the master boot record" \
            "partition" "Install boot loader to the root partition" \
            "none" "Do not install a boot loader at all" \
            2> /tmp/prompt
        # check for cancellation
        if [ $? -ne 0 ]; then
            CANCEL=yes
            break
        fi
        CHOICE=$(cat /tmp/prompt)
        pgilog "User chose to install boot loader to \"$CHOICE\"."

        case "$CHOICE" in
            mbr|partition)
                INSTALL_BOOTLOADER=yes
                ;;
            none)
                ;;
        esac
    fi
fi

if [ -z "$CANCEL" ]; then
    # initialize the system
    rm -f /tmp/messages /tmp/setup-base-text-done

    if [ -e "$EXTRA_PACKAGES" ]; then
      EXTRA_PKG_ARG="--extra-packages $EXTRA_PACKAGES"
    fi

    if [ -e "$UNWANTED_PACKAGES" ]; then
      UNWANTED_PKG_ARG="--unwanted-packages $UNWANTED_PACKAGES"
    fi

    pgilog -n "Running \"python $LIVE/share/installer/setup-base-text.py --target $TARGET --mirror $MIRROR --suite $SUITE $EXTRA_PKG_ARG $UNWANTED_PKG_ARG --swaps $SWAPPART $MNTPNTS\" in the background..."

    ( if python $LIVE/share/installer/setup-base-text.py --target $TARGET \
          --mirror $MIRROR --suite $SUITE $EXTRA_PKG_ARG $UNWANTED_PKG_ARG \
          --swaps $SWAPPART $MNTPNTS; then
          echo "$?" > /tmp/setup-base-text-done
          echo "Initialization successful.  Press Enter to continue."
      else
          exitstatus=$?
          echo "$exitstatus" > /tmp/setup-base-text-done
          pgilog "setup-base-text.py failed with exit status $exitstatus."
          echo "Initialization failed.  Press Enter to cancel."
      fi ) > /tmp/messages &

    pgilog "forked pid $!."

    # wait for /tmp/messages to exist before attempting to drive a tailbox
    # dialog with it
    while [ ! -e /tmp/messages ]; do
        sleep 1
    done

    $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
      --title "Initializing system..." \
      --tailbox /tmp/messages 19 74

    if [ ! -e /tmp/setup-base-text-done ]; then
        echo -n "System initialization not finished yet.  Please wait."
        while [ ! -e /tmp/setup-base-text-done ]; do
            sleep 1
            echo -n "."
        done
    fi

    if [ "$(cat /tmp/setup-base-text-done)" != "0" ]; then
        CANCEL=yes
    fi
fi

if [ -z "$CANCEL" ]; then
    if [ -n "$INSTALL_BOOTLOADER" ]; then
    # XXX: arch-specific tests shouldn't be done here
        if [ "$ARCH" = "ia64" ]; then
            argument="$EFIPART"
        else
            if [ "$CHOICE" = "mbr" ]; then
                argument="$ROOTDISK"
            else
                argument="$ROOTPART"
            fi
        fi

        pgilog "Installing boot loader to $argument."

        rm -f /tmp/messages /tmp/setup-bootloader-text-done

        pgilog -n "Running \"python $LIVE/share/installer/setup-bootloader-text.py --root $ROOTPART --target $TARGET $argument\" in the background..."

        ( if python $LIVE/share/installer/setup-bootloader-text.py \
              --root $ROOTPART --target $TARGET $argument; then
              echo "$?" > /tmp/setup-bootloader-text-done
              echo "Boot loader installed.  Press Enter to continue."
          else
              exitstatus=$?
              echo "$exitstatus" > /tmp/setup-bootloader-text-done
              pgilog "setup-bootloader-text.py failed with exit status $exitstatus."
              echo "Boot loader installation failed.  Press Enter to cancel."
          fi ) > /tmp/messages &

        pgilog "forked pid $!."

        # wait for /tmp/messages to exist before attempting to drive a tailbox
        # dialog with it
        while [ ! -e /tmp/messages ]; do
            sleep 1
        done

        $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
          --title "Installing boot loader..." \
          --tailbox /tmp/messages 19 74

        if [ ! -e /tmp/setup-bootloader-text-done ]; then
            echo -n "Boot loader installation not finished yet.  Please wait."
            while [ ! -e /tmp/setup-bootloader-text-done ]; do
                sleep 1
                echo -n "."
            done
        fi

        if [ "$(cat /tmp/setup-bootloader-text-done)" != "0" ]; then
            CANCEL=yes
        fi
    fi

    # get congratulations message
    python $LIVE/share/installer/get-message.py finish | sed 's/"/\\"/g' \
        > /tmp/congrat-msg
    if [ ! -s /tmp/congrat-msg ]; then
        cat > /tmp/congrat-msg << EOF
Congratulations, the system packages have been installed.

The system will now boot into your new Debian GNU/Linux
installation.
EOF
    fi

    # congratulations
    $DIALOG --cr-wrap --backtitle "PGI Debian Installer" \
        --title "System Installed" \
        --textbox /tmp/congrat-msg $(calc_textbox_size /tmp/congrat-msg)
    # no need to check for cancellation on this dialog; it's a bit late
fi

# clear the screen if we're not debugging
if [ -z "$PGIDEBUG" ]; then
    clear
fi

# vim:ai:et:sts=4:sw=4:tw=0:
