#! /usr/bin/python

# $Progeny: setup-base-text.py,v 1.22 2002/06/12 19:45:36 branden Exp $

import re
import sys

import pgi.base
import pgi.common

from getopt import getopt, GetoptError
from pgi.common import TRUE, FALSE, EX_USAGE, _, warn

def print_statusline(line, msgtype = None):
    if line:
        if msgtype == "install_kernel":
            if (re.match("^\./.*", line)):
                # tell user that and what we're copying
                line = "Copying %s" % (line,)

        elif msgtype == "install_packages":
            if (re.match("^I: .*", line)):
                # strip off the leading "I: "
                line = re.sub(r'^I: ', '', line)

        # don't let message lines get too long
        line = pgi.common.crunchline(line, 70)
        sys.stdout.write("%s\n" % (line,))
        sys.stdout.flush()

###############################################################################
# getopt

extra_packages = None
unwanted_packages = None
mirror = "http://archive.progeny.com/debian"
suite = "stable"
swaps = []
target = "/target"

try:
    (opts, args) = getopt(sys.argv[1:], "",
                          ["extra-packages=",
                           "unwanted-packages=",
                           "mirror=",
                           "suite=",
                           "swaps=",
                           "target="])
except GetoptError:
    sys.exit(EX_USAGE)

for (o, a) in opts:
    if o == "--extra-packages":
        extra_packages = a
        pgi.common.msg("Got extra packages file: %s\n" % (extra_packages,))
    elif o == "--unwanted-packages":
        unwanted_packages = a
        pgi.common.msg("Got unwanted packages file: %s\n" % (unwanted_packages,))
    elif o == "--mirror":
        mirror = a
    elif o == "--suite":
        suite = a
    elif o == "--swaps":
        for i in re.split(r"\s+", a):
            swaps.append(i)
    elif o == "--target":
        target = a

argc = len(args)
if argc == 0 or (argc % 3 != 0):
    sys.exit(EX_USAGE)

mounts = {}
i = 0
while i < argc:
    mounts[args[i]] = [args[i + 1], args[i + 2]]
    i = i + 3

###############################################################################
# Main

print "Mounting target filesystem(s)."
if not pgi.base.mount(mounts, target):
    sys.exit(1)

print "Installing packages."
if not pgi.base.install_packages(suite, mirror, target, extra_packages,
                                 unwanted_packages, print_statusline,
                                 "install_packages"):
    sys.exit(2)

print "Creating /etc/fstab filesystem information file."
if not pgi.base.mkfstab(mounts, swaps, target):
    sys.exit(3)

print "Installing the kernel."
if not pgi.base.install_kernel(target, print_statusline,
                               "install_kernel"):
    sys.exit(4)

# vim:ai:et:sts=4:sw=4:tw=0:
