#!/bin/sh
#
# $Progeny: detectmodules.sh,v 1.23 2002/05/22 21:32:47 branden Exp $
#
# Echo module names found from the discover database as well as those in the
# PCMCIA modules package.  These should be output sorted by priority,
# most important modules first.

report_module() {
    if [ `find /lib/modules/$version -name $1.o -print | wc -l` -gt 0 ]; then
        echo $1
    fi
}

if [ "$#" -lt "1" ] ; then
    echo "Usage: $0 kernel_version [ module ... ]" >&2
    exit 1
fi

version=$1
shift

if [ -n "$1" ]; then
    base_modules=$@
fi

# from the environment; migrate to getopt someday
: ${discover_classes:=cdrom|ether|scsi|usb}

# basic set of modules
if [ -z "$base_modules" ]; then
    if expr "$version" : "^2\.2" > /dev/null 2>&1; then
        base_modules="af_packet isofs nfs nls_cp437 ide-scsi mousedev vfat \
                      unix agpgart sd_mod sr_mod usb-storage i82365 ds"
    elif expr "$version" : "^2\.4" > /dev/null 2>&1; then
        base_modules="af_packet isofs nfs nls_cp437 ide-probe-mod ide-disk \
                      ide-scsi mousedev vfat unix agpgart sd_mod sr_mod \
                      usb-storage i82365 ds efivars"
    else
        echo "$0: error: no base modules defined for unrecognized kernel $version." >&2
        exit 1
    fi
fi

# list basic modules first
for module in $base_modules
do
    report_module $module
done

# pull the initial module listing out of the discover database based on it
# being a cdrom|scsi module
# XXX: When PGI supports a boot medium other than CD/DVD-ROM again, we will
# have to add any module type needed to access the medium with the modules
# archive (e.g., ethernet), or tell the person using pgi-build to mark
# those modules as essential.
for module in $(awk "{ if (match(\$2, \"$discover_classes\") \
     && !match(\$3, \"(ignore|unknown)\")) print $3 }" \
     /usr/share/discover/*.lst | sort -u)
do
    report_module $module
done

# Fix bugs/omissions in discover database:
report_module pcnet32

# add PCMCIA modules
if dpkg --listfiles pcmcia-modules-$version > /dev/null 2>&1; then
    for file in $(dpkg --listfiles pcmcia-modules-$version 2>/dev/null | \
                  grep /lib/modules)
    do
        if [ ! -d $file ] ; then
            report_module $(basename $file) | sed 's/\.o//'
        fi
    done
else
    echo "$0: note: no pcmcia-modules-$version package installed; not including PCMCIA modules on initrd" >&2
fi

# vim:ai:et:sts=4:sw=4:tw=0:
